<?php
/**
 * General functions used by Link Mananager 2 plugin.
 *
 * (c)2012 mrdragonraaar.com
 */
if (!defined('IN_GS')) { die('you cannot load this page directly.'); }

// plugin template path
if (!defined('LM2_TEMPLATE_PATH')) {
define('LM2_TEMPLATE_PATH', LM2_PLUGIN_PATH . 'template/');
}

require_once('lib/LinkManager/LinkManager.php');

/**
 * Main back-end plugin function.
 * Determine what action to take.
 */
function lm2_main()
{
	global $lm2;

	if ($lm2::is_link_page())
	{
		if ($lm2::is_post_link())
		{
			$lm2->add_post_link();
			if ($lm2->save_links())
				lm2_admin_message(true);
			else
				lm2_admin_message(false);
			lm2_admin_link_panel();
		}
		elseif ($lm2::is_post_link_order())
		{
			$lm2->order_post_links();
			if ($lm2->save_links())
				lm2_admin_message(true);
			else
				lm2_admin_message(false);
			lm2_admin_link_panel();
		}
		elseif ($lm2::is_edit_page())
		{
			lm2_edit_link_panel();
		}
		elseif ($lm2::is_delete_page())
		{
			$lm2->delete_link();
			if ($lm2->save_links())
				lm2_admin_message(true);
			else
				lm2_admin_message(false);
			lm2_admin_link_panel();
		}
		elseif ($lm2::is_undo_page())
		{
			if ($lm2->undo_links())
				lm2_admin_message(true);
			else
				lm2_admin_message(false);
			lm2_admin_link_panel();
		}
		else
		{
			lm2_admin_link_panel();
		}
	}

	if ($lm2::is_category_page())
	{
		if ($lm2::is_post_category())
		{
			$lm2->add_post_category();
			if ($lm2->save_categories())
				lm2_admin_message(true);
			else
				lm2_admin_message(false);
			lm2_admin_category_panel();
		}
		elseif ($lm2::is_edit_page())
		{
			lm2_edit_category_panel();
		}
		elseif ($lm2::is_delete_page())
		{
			$lm2->delete_category();
			if ($lm2->save_categories())
				lm2_admin_message(true);
			else
				lm2_admin_message(false);
			lm2_admin_category_panel();
		}
		elseif ($lm2::is_undo_page())
		{
			if ($lm2->undo_categories())
				lm2_admin_message(true);
			else
				lm2_admin_message(false);
			lm2_admin_category_panel();
		}
		else
		{
			lm2_admin_category_panel();
		}
	}
}

/**
 * Display admin panel for links.
 */
function lm2_admin_link_panel()
{
	global $lm2;

	include(LM2_TEMPLATE_PATH . 'admin_link_panel.php');
}

/**
 * Display admin panel for edit link.
 */
function lm2_edit_link_panel()
{
	global $lm2;

	include(LM2_TEMPLATE_PATH . 'edit_link_panel.php');
}

/**
 * Display admin panel for categories.
 */
function lm2_admin_category_panel()
{
	global $lm2;

	include(LM2_TEMPLATE_PATH . 'admin_category_panel.php');
}

/**
 * Display admin panel for edit category.
 */
function lm2_edit_category_panel()
{
	global $lm2;

	include(LM2_TEMPLATE_PATH . 'edit_category_panel.php');
}

/**
 * Display message on admin panel.
 * @param $success denotes whether to display success message.
 */
function lm2_admin_message($success)
{
	global $lm2;

	include(LM2_TEMPLATE_PATH . 'admin_message.php');
}


/**
 * Front-end plugin function.
 * Display list of links.
 * @param $category_id category id
 * @param $display True to echo, false to return
 */
function get_links($category_id = null, $display = true)
{
	global $lm2;

	if (!$display) { ob_start(); }
	include(LM2_TEMPLATE_PATH . 'links_html.php');
	if (!$display) { return ob_get_clean(); }
}

/**
 * Front-end plugin function.
 * Display category name.
 * @param $category_id category id
 * @param $display True to echo, false to return
 */
function get_link_category($category_id, $display = true)
{
	global $lm2;

	if ($display)
		echo $lm2->category_name($category_id);
	else
		return $lm2->category_name($category_id);
}

/**
 * Front-end plugin function.
 * Get link data.
 * @param $category_id category id
 */
function link_data($category_id = null)
{
	global $lm2;

	return isset($category_id) ? 
	   $lm2->links_by_category($category_id) :
	   $lm2->links();
}

/**
 * Front-end plugin function.
 * Get link category data.
 * @param $category_id category id
 */
function link_category_data()
{
	global $lm2;

	return $lm2->categories();
}

/**
 * Front-end plugin function for filter.
 * Insert list of links into content.
 * @param $content page content
 */
function lm2_filter($content)
{
	if (preg_match('/\(%\s*links\s*%\)/', $content))
	{
		$html = get_links(null, false);
		$content = preg_replace('/\(%\s*links\s*%\)/', $html, $content);
	}

	if (preg_match('/\(%\s*links:(\d*)\s*%\)/', $content, $matches))
	{
		$content = preg_replace('/\(%\s*links:(\d*)\s*%\)/e', 'get_links(intval($1), false)', $content);
	}

	if (preg_match('/\(%\s*link_category:(\d*)\s*%\)/', $content, $matches))
	{
		$content = preg_replace('/\(%\s*link_category:(\d*)\s*%\)/e', 'get_link_category(intval($1), false)', $content);
	}

	return $content;
}

/**
 * Front-end plugin function. Deprecated.
 * Display list of links.
 * @param $category_id category id
 */
function lm2_list_links($category_id = '')
{
	get_links($category_id);
}

/**
 * Front-end plugin function. Deprecated.
 * Display category name.
 * @param $category_id category id
 */
function lm2_category($category_id = '')
{
	get_link_category($category_id);
}

?>
