<?php
/**
 * General class and methods used by Link Mananager 2 plugin.
 *
 * (c)2012 mrdragonraaar.com
 */
if (!defined('IN_GS')) { die('you cannot load this page directly.'); }

// plugin link data path
if (!defined('LM2_LINK_DATA')) {
define('LM2_LINK_DATA', GSDATAOTHERPATH . 'links.xml');
}
// plugin category data path
if (!defined('LM2_CATEGORY_DATA')) {
define('LM2_CATEGORY_DATA', GSDATAOTHERPATH . 'link_categories.xml');
}
// plugin backup link data path
if (!defined('LM2_LINK_BACKUP')) {
define('LM2_LINK_BACKUP', GSBACKUPSPATH . 'other/links.xml');
}
// plugin backup category data path
if (!defined('LM2_CATEGORY_BACKUP')) {
define('LM2_CATEGORY_BACKUP', GSBACKUPSPATH . 'other/link_categories.xml');
}

/**
 * Class and methods for reading and writing link and category data.
 *
 * @author Adrian D. Elgar
 */
class LinkManager
{
	// list of links
	private $_links = array();
	// list of categories
	private $_categories = array();

	/**
	 * Get list of links.
	 * @return list of links
	 */
	public function links()
	{
		return empty($this->_links) ? 
		   $this->_load_links() : $this->_links;
	}

	/**
	 * Get links for given category id.
	 * @param $category_id category id
	 * @return list of links
	 */
	public function links_by_category($category_id)
	{
		$links = array();
		foreach ($this->links() as $id => $link)
		{
			if ($link->category == $category_id)
			{
				$links[$id] = $link;
			}
		}

		return $links;
	}

	/**
	 * Get link for given id.
	 * @param $id link id
	 * @return link
	 */
	public function link($id)
	{
		$links = $this->links();

		return isset($links[$id]) ? $links[$id] : null;
	}

	/**
	 * Get link for given url.
	 * @param $url link url
	 * @return link
	 */
	public function link_by_url($url)
	{
		foreach ($this->links() as $link)
		{
			if (!strcmp($url, $link->url))
			{
				return $link;
			}
		}

		return null;
	}

	/**
	 * Create a new link.
	 * @param $name link name
	 * @param $url link url
	 * @param $target link target
	 * @param $desc link description
	 * @param $category link category id
	 * @param $icon link icon url
	 * @return new link
	 */
	public function new_link($name, $url, $target, $desc, 
	   $category, $icon)
	{
		return $this->_links[] = new Link($name, $url, 
		   $target, $desc, $category, $icon);
	}

	/**
	 * Add a link.
	 * @param $id link id
	 * @param $name link name
	 * @param $url link url
	 * @param $target link target
	 * @param $desc link description
	 * @param $category link category id
	 * @param $icon link icon url
	 * @return link
	 */
	public function add_link($id, $name, $url, $target, $desc, 
	   $category, $icon)
	{
		return $this->_links[$id] = new Link($name, $url, 
		   $target, $desc, $category, $icon);
	}

	/**
	 * Remove link for given id.
	 * @param $id link id
	 */
	public function remove_link($id)
	{
		$link = $this->link($id);

		if (isset($link))
			unset($this->_links[$id]);
	}

	/**
	 * Load list of links.
	 * @return list of links
	 */
	private function _load_links()
	{
		$data = @getXML(LM2_LINK_DATA);

		if (!empty($data))
		{
			foreach ($data->children() as $item)
			{
				$id = intval($item->id);
				$name = cl($item->name);
				$url = strval($item->url);
				$target = cl($item->target);
				$desc = cl($item->desc);
				$category = intval($item->category);
				$icon = strval($item->icon);
				$this->add_link($id, $name, $url, 
				   $target, $desc, $category, $icon);
			}
		}

		return $this->_links;
	}

	/**
	 * Save list of links.
	 * @return true, if successful
	 */
	public function save_links()
	{
		@copy(LM2_LINK_DATA, LM2_LINK_BACKUP);
		$xml = new SimpleXMLExtended('<?xml version="1.0" encoding="UTF-8"?><channel></channel>');

		foreach ($this->links() as $id => $link)
		{
			$item = $xml->addChild('item');
			$elem = $item->addChild('id');
			$elem->addCData($id);
			$elem = $item->addChild('name');
			$elem->addCData($link->name);
			$elem = $item->addChild('url');
			$elem->addCData($link->url);
			$elem = $item->addChild('target');
			$elem->addCData($link->target);
			$elem = $item->addChild('desc');
			$elem->addCData($link->desc);
			$elem = $item->addChild('category');
			$elem->addCData($link->category);
			$elem = $item->addChild('icon');
			$elem->addCData($link->icon);
		}

		return @XMLsave($xml, LM2_LINK_DATA);
	}

	/**
	 * Get list of categories.
	 * @return list of categories
	 */
	public function categories()
	{
		return empty($this->_categories) ? 
		   $this->_load_categories() : $this->_categories;
	}

	/**
	 * Get category for given id.
	 * @param $id category id
	 * @return category
	 */
	public function category($id)
	{
		$categories = $this->categories();

		return isset($categories[$id]) ? $categories[$id] : null;
	}

	/**
	 * Get category name for given id.
	 * @param $id category id
	 * @return category name
	 */
	public function category_name($id)
	{
		$category = $this->category($id);

		return isset($category) ? $category->name : null;
	}

	/**
	 * Create a new category.
	 * @param $name category name
	 * @return new category
	 */
	public function new_category($name)
	{
		return $this->_categories[] = new Category($name);
	}

	/**
	 * Add a category.
	 * @param $id category id
	 * @param $name category name
	 * @return category
	 */
	public function add_category($id, $name)
	{
		return $this->_categories[$id] = new Category($name);
	}

	/**
	 * Remove a category.
	 * @param $id category id
	 */
	public function remove_category($id)
	{
		$category = $this->category($id);

		if (isset($category))
			unset($this->_categories[$id]);
	}

	/**
	 * Load list of categories.
	 * @return list of categories
	 */
	private function _load_categories()
	{
		$data = @getXML(LM2_CATEGORY_DATA);

		if (!empty($data))
		{
			foreach ($data->children() as $item)
			{
				$id = intval($item->id);
				$name = cl($item->name);
				$this->add_category($id, $name);
			}
		}

		return $this->_categories;
	}

	/**
	 * Save list of categories.
	 * @return true, if successful
	 */
	public function save_categories()
	{
		@copy(LM2_CATEGORY_DATA, LM2_CATEGORY_BACKUP);
		$xml = new SimpleXMLExtended('<?xml version="1.0" encoding="UTF-8"?><channel></channel>');

		foreach ($this->categories() as $id => $category)
		{
			$item = $xml->addChild('item');
			$elem = $item->addChild('id');
			$elem->addCData($id);
			$elem = $item->addChild('name');
			$elem->addCData($category->name);
		}

		return @XMLsave($xml, LM2_CATEGORY_DATA);
	}

	/**
	 * Get url for plugin.
	 * @return plugin url
	 */
	static public function plugin_url()
	{
		return 'load.php?id=' . LM2_PLUGIN_ID;
	}

	/**
	 * Get url for plugin links page.
	 * @return plugin links url
	 */
	static public function link_url()
	{
		return self::plugin_url() . '&amp;links';
	}

	/**
	 * Test if current page is plugin links page.
	 * @return True if plugin links url
	 */
	static public function is_link_page()
	{
		return isset($_GET['links']);
	}

	/**
	 * Get url for plugin categories page.
	 * @return plugin categories url
	 */
	static public function category_url()
	{
		return self::plugin_url() . '&amp;categories';
	}

	/**
	 * Test if current page is plugin categories page.
	 * @return True if plugin categories url
	 */
	static public function is_category_page()
	{
		return isset($_GET['categories']);
	}

	/**
	 * Get url for plugin command.
	 * @param $url base url
	 * @param $command plugin command
	 * @param $id command id
	 * @return plugin command url
	 */
	static private function command_url($url, $command, $id = null)
	{
		return $url . '&amp;' . $command . (isset($id) ? ('=' . $id) : '');
	}

	/**
	 * Get url for plugin edit command.
	 * @param $url base url
	 * @param $id edit id
	 * @return plugin edit url
	 */
	static private function edit_url($url, $id = null)
	{
		return self::command_url($url, 'edit', $id);
	}

	/**
	 * Get url for plugin links edit page.
	 * @param $id link id
	 * @return plugin links edit url
	 */
	static public function link_edit_url($id = null)
	{
		return self::edit_url(self::link_url(), $id);
	}

	/**
	 * Get url for plugin categories edit page.
	 * @param $id category id
	 * @return plugin categories edit url
	 */
	static public function category_edit_url($id = null)
	{
		return self::edit_url(self::category_url(), $id);
	}

	/**
	 * Test if current command is plugin edit command.
	 * @return True if plugin edit url
	 */
	static public function is_edit_page()
	{
		return isset($_GET['edit']);
	}

	/**
	 * Get edit id.
	 * @return edit id
	 */
	static public function edit_id()
	{
		return self::is_edit_page() ? $_GET['edit'] : null;
	}

	/**
	 * Get edit link.
	 * @return edit link
	 */
	public function edit_link()
	{
		return $this->link(self::edit_id());
	}

	/**
	 * Get edit category.
	 * @return edit category
	 */
	public function edit_category()
	{
		return $this->category(self::edit_id());
	}

	/**
	 * Get url for plugin delete command.
	 * @param $url base url
	 * @param $id delete id
	 * @return plugin delete url
	 */
	static private function delete_url($url, $id = null)
	{
		return self::command_url($url, 'delete', $id);
	}

	/**
	 * Get url for plugin links delete.
	 * @param $id link id
	 * @return plugin links delete url
	 */
	static public function link_delete_url($id = null)
	{
		return self::delete_url(self::link_url(), $id);
	}

	/**
	 * Get url for plugin categories delete.
	 * @param $id link id
	 * @return plugin categories delete url
	 */
	static public function category_delete_url($id = null)
	{
		return self::delete_url(self::category_url(), $id);
	}

	/**
	 * Test if current command is plugin delete command.
	 * @return True if plugin delete url
	 */
	static public function is_delete_page()
	{
		return isset($_GET['delete']);
	}

	/**
	 * Get delete id.
	 * @return delete id
	 */
	static public function delete_id()
	{
		return self::is_delete_page() ? $_GET['delete'] : null;
	}

	/**
	 * Get delete link.
	 * @return delete link
	 */
	public function delete_link()
	{
		return $this->remove_link(self::delete_id());
	}

	/**
	 * Get delete category.
	 * @return delete category
	 */
	public function delete_category()
	{
		return $this->remove_category(self::delete_id());
	}

	/**
	 * Get url for plugin cancel command.
	 * @param $url base url
	 * @return plugin cancel url
	 */
	static private function cancel_url($url)
	{
		return self::command_url($url, 'cancel');
	}

	/**
	 * Get url for plugin links cancel.
	 * @return plugin links cancel url
	 */
	static public function link_cancel_url()
	{
		return self::cancel_url(self::link_url());
	}

	/**
	 * Get url for plugin categories cancel.
	 * @return plugin categories cancel url
	 */
	static public function category_cancel_url()
	{
		return self::cancel_url(self::category_url());
	}

	/**
	 * Get url for plugin undo command.
	 * @param $url base url
	 * @return plugin undo url
	 */
	static private function undo_url($url)
	{
		return self::command_url($url, 'undo');
	}

	/**
	 * Get url for plugin links undo.
	 * @return plugin links undo url
	 */
	static public function link_undo_url()
	{
		return self::undo_url(self::link_url());
	}

	/**
	 * Get url for plugin categories undo.
	 * @return plugin categories undo url
	 */
	static public function category_undo_url()
	{
		return self::undo_url(self::category_url());
	}

	/**
	 * Test if current command is plugin undo command.
	 * @return True if plugin undo url
	 */
	static public function is_undo_page()
	{
		return isset($_GET['undo']);
	}

	/**
	 * Undo changes to links data.
	 * @return True if undo successful
	 */
	public function undo_links()
	{
		return copy(LM2_LINK_BACKUP, LM2_LINK_DATA);
	}

	/**
	 * Undo changes to categories data.
	 * @return True if undo successful
	 */
	public function undo_categories()
	{
		return copy(LM2_CATEGORY_BACKUP, LM2_CATEGORY_DATA);
	}

	/**
	 * Test if edit link form has been posted.
	 * @return True if link form has been posted
	 */
	static public function is_post_link()
	{
		return isset($_POST['link']);
	}

	/**
	 * Get link id from edit link form.
	 * @return link id
	 */
	static public function post_link_id()
	{
		return isset($_POST['link-id']) ? 
		   intval($_POST['link-id']) : null;
	}

	/**
	 * Get link name from edit link form.
	 * @return link name
	 */
	static public function post_link_name()
	{
		return isset($_POST['link-name']) ? 
		   safe_slash_html($_POST['link-name']) : null;
	}

	/**
	 * Get link url from edit link form.
	 * @return link url
	 */
	static public function post_link_url()
	{
		return isset($_POST['link-url']) ? 
		   $_POST['link-url'] : null;
	}

	/**
	 * Get link target from edit link form.
	 * @return link target
	 */
	static public function post_link_target()
	{
		return isset($_POST['link-target']) ? 
		   $_POST['link-target'] : null;
	}

	/**
	 * Get link description from edit link form.
	 * @return link description
	 */
	static public function post_link_desc()
	{
		return isset($_POST['link-desc']) ? 
		   $_POST['link-desc'] : null;
	}

	/**
	 * Get link catergory id from edit link form.
	 * @return link category id
	 */
	static public function post_link_category()
	{
		return isset($_POST['link-category']) ? 
		   intval($_POST['link-category']) : -1;
	}

	/**
	 * Get link icon url from edit link form.
	 * @return link icon url
	 */
	static public function post_link_icon()
	{
		return isset($_POST['link-icon']) ? 
		   $_POST['link-icon'] : null;
	}

	/**
	 * Add link from edit link form.
	 */
	public function add_post_link()
	{
		if (self::is_post_link())
		{
			return ($this->links() && 
			   $this->link(self::post_link_id())) ? 
			   $this->add_link(self::post_link_id(), 
			      self::post_link_name(), self::post_link_url(),
			      self::post_link_target(), self::post_link_desc(),
			      self::post_link_category(), self::post_link_icon()) : 
			   $this->new_link(self::post_link_name(), self::post_link_url(),
			      self::post_link_target(), self::post_link_desc(),
			      self::post_link_category(), self::post_link_icon());
		}

		return null;
	}

	/**
	 * Test if edit category form has been posted.
	 * @return True if category form has been posted
	 */
	static public function is_post_category()
	{
		return isset($_POST['category']);
	}

	/**
	 * Get category id from edit category form.
	 * @return category id
	 */
	static public function post_category_id()
	{
		return isset($_POST['category-id']) ? 
		   intval($_POST['category-id']) : null;
	}

	/**
	 * Get category name from edit category form.
	 * @return category name
	 */
	static public function post_category_name()
	{
		return isset($_POST['category-name']) ? 
		   safe_slash_html($_POST['category-name']) : null;
	}

	/**
	 * Add category from edit category form.
	 */
	public function add_post_category()
	{
		if (self::is_post_category())
		{
			return ($this->categories() && 
			   $this->category(self::post_category_id())) ? 
			   $this->add_category(self::post_category_id(), 
			      self::post_category_name()) : 
			   $this->new_category(self::post_category_name());
		}

		return null;
	}

	/**
	 * Test if change link order form has been posted.
	 * @return True if change link order form has been posted
	 */
	static public function is_post_link_order()
	{
		return isset($_POST['order']);
	}

	/**
	 * Get link order from change link order form.
	 * @return link order
	 */
	static public function post_link_order()
	{
		return isset($_POST['link-order']) ? 
		   $_POST['link-order'] : null;
	}

	/**
	 * Change link order.
	 */
	public function order_post_links()
	{
		if (self::is_post_link_order() &&
		   self::post_link_order())
		{
			$links = array();
			$link_order = explode(',', self::post_link_order());

			foreach ($link_order as $url)
			{
				if ($url)
				{
					$link = $this->link_by_url($url);
					if (isset($link))
					{
						$links[] = $link;
					}
				}
			}

			$this->_links = $links;
		}
	}

}


/**
 * Define a link.
 *
 * @author Adrian D. Elgar
 */
class Link
{
	public $id;
	public $name;
	public $url;
	public $target;
	public $desc;
	public $category;
	public $icon;

	function __construct($name, $url, $target, 
	   $desc, $category, $icon)
	{
		$this->name = $name;
		$this->url = $url;
		$this->target = $target;
		$this->desc = $desc;
		$this->category = $category;
		$this->icon = $icon;
	}
}


/**
 * Define a category.
 *
 * @author Adrian D. Elgar
 */
class Category
{
	public $id;
	public $name;

	function __construct($name)
	{
		$this->name = $name;
	}
}


// create instance
$lm2 = new LinkManager();

?>
